@@ -9,7 +9,10 @@ use swc_ecma_utils::{
 };
 use swc_ecma_visit::VisitMutWith;
 
-use super::{util::MultiReplacer, Optimizer};
+use super::{
+    util::{MultiReplacer, MultiReplacerMode},
+    Optimizer,
+};
 use crate::{
     compress::optimize::{util::Remapper, Ctx},
     debug::dump,
@@ -262,7 +265,7 @@ where
         n.visit_mut_with(&mut MultiReplacer::new(
             &mut vars,
             false,
-            false,
+            MultiReplacerMode::Normal,
             &mut self.changed,
         ));
     }
@@ -437,18 +440,9 @@ where
                                 exprs.push(arg.expr.take());
                             }
                         }
-                        body.visit_mut_with(&mut MultiReplacer::new(
-                            &mut self.simple_functions,
-                            true,
-                            true,
-                            &mut self.changed,
-                        ));
-                        body.visit_mut_with(&mut MultiReplacer::new(
-                            &mut self.vars_for_inlining,
-                            false,
-                            false,
-                            &mut self.changed,
-                        ));
+                        if self.vars.inline_with_multi_replacer(body) {
+                            self.changed = true;
+                        }
                         body.visit_mut_with(&mut Remapper { vars: remap });
                         exprs.push(body.take());
 
@@ -549,10 +543,6 @@ where
         }
     }
 
-    fn has_pending_inline_for(&self, id: &Id) -> bool {
-        self.lits.contains_key(id) || self.vars_for_inlining.contains_key(id)
-    }
-
     fn is_return_arg_simple_enough_for_iife_eval(&self, e: &Expr) -> bool {
         match e {
             Expr::Lit(..) | Expr::Ident(..) => true,
@@ -632,7 +622,7 @@ where
                         ..
                     }) => true,
                     Pat::Ident(id) => {
-                        if self.has_pending_inline_for(&id.to_id()) {
+                        if self.vars.has_pending_inline_for(&id.to_id()) {
                             return true;
                         }
 
@@ -729,18 +719,9 @@ where
             }
         }
 
-        body.visit_mut_with(&mut MultiReplacer::new(
-            &mut self.simple_functions,
-            true,
-            true,
-            &mut self.changed,
-        ));
-        body.visit_mut_with(&mut MultiReplacer::new(
-            &mut self.vars_for_inlining,
-            false,
-            false,
-            &mut self.changed,
-        ));
+        if self.vars.inline_with_multi_replacer(body) {
+            self.changed = true;
+        }
         body.visit_mut_with(&mut Remapper { vars: remap });
 
         {
