@@ -34,6 +34,14 @@ where
             return;
         }
 
+        if stmt
+            .cases
+            .iter()
+            .any(|case| matches!(case.test.as_deref(), Some(Expr::Update(..))))
+        {
+            return;
+        }
+
         let matching_case = stmt.cases.iter_mut().position(|case| {
             case.test
                 .as_ref()
@@ -117,7 +125,16 @@ where
                     _ => !found_break,
                 });
 
-                stmts.append(&mut case.cons);
+                for case_stmt in case.cons.take() {
+                    match case_stmt {
+                        Stmt::Decl(Decl::Var(v)) if v.decls.iter().all(|v| v.init.is_none()) => {
+                            var_ids.extend(v.decls)
+                        }
+                        _ => {
+                            stmts.push(case_stmt);
+                        }
+                    }
+                }
                 if found_break {
                     break;
                 }
@@ -235,7 +252,7 @@ where
     /// If a case ends with break but content is same with the consequtive case
     /// except the break statement, we merge them.
     fn merge_cases_with_same_cons(&mut self, cases: &mut Vec<SwitchCase>) {
-        let stop_pos = cases
+        let mut stop_pos_opt = cases
             .iter()
             .position(|case| matches!(case.test.as_deref(), Some(Expr::Update(..))));
 
@@ -245,8 +262,14 @@ where
                 continue;
             }
 
-            if let Some(stop_pos) = stop_pos {
-                if li > stop_pos {
+            if let Some(stop_pos) = stop_pos_opt {
+                if li == stop_pos {
+                    // Look for next stop position
+                    stop_pos_opt = cases
+                        .iter()
+                        .skip(li)
+                        .position(|case| matches!(case.test.as_deref(), Some(Expr::Update(..))))
+                        .map(|v| v + li);
                     continue;
                 }
             }
